@@ -58,7 +58,6 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <ctype.h> // for isspace()
 
 #include <unistd.h>  // for getopt()
 #include <getopt.h> // for getopt_long()
@@ -66,7 +65,7 @@
 #include "igd.h"
 #include "nondominated.h"
 
-#define CMDLINE_COPYRIGHT_YEARS "2016"
+#define CMDLINE_COPYRIGHT_YEARS "2016-2024"
 #define CMDLINE_AUTHORS "Manuel Lopez-Ibanez  <manuel.lopez-ibanez@manchester.ac.uk>\n" \
     "Leonardo C. T. Bezerra <leo.tbezerra@gmail.com>\n"
 #include "cmdline.h"
@@ -76,7 +75,7 @@ static unsigned int exponent_p = 1;
 static bool gd = false;
 static bool igd = false;
 static bool gdp = false;
-static bool igdp = true;
+static bool igdp = false;
 static bool igdplus = false;
 static bool hausdorff = false;
 
@@ -95,39 +94,38 @@ static void usage(void)
 "Options:\n"
 OPTION_HELP_STR
 OPTION_VERSION_STR
-" -v, --verbose        print some information (time, number of points, etc.) \n"
+" -v, --verbose       print some information (time, number of points, etc.) \n"
 OPTION_QUIET_STR
-"   , --gd             %s report classical GD\n"
-"   , --igd            %s report classical IGD\n"
-"   , --gd-p           %s report GD_p (p=1 by default)\n"
-"   , --igd-p          %s report IGD_p (p=1 by default)\n"
-"   , --igd-plus       %s report IGD+\n"
-"   , --hausdorff      %s report avg Hausdorff distance = max (GD_p, IGD_p)\n"
-" -a, --all            compute everything\n"
-" -p,                  exponent that averages the distances\n"
+"   , --gd            report classical GD\n"
+"   , --igd           report classical IGD\n"
+"   , --gd-p          report GD_p (p=1 by default)\n"
+"   , --igd-p         (default) report IGD_p (p=1 by default)\n"
+"   , --igd-plus      report IGD+\n"
+"   , --hausdorff     report avg Hausdorff distance = max (GD_p, IGD_p)\n"
+" -a, --all           compute everything\n"
+" -p,                 exponent that averages the distances\n"
 " -r, --reference FILE file that contains the reference set                  \n"
 OPTION_OBJ_STR
+OPTION_MAXIMISE_STR
 " -s, --suffix=STRING Create an output file for each input file by appending\n"
 "                     this suffix. This is ignored when reading from stdin. \n"
 "                     If missing, output is sent to stdout.                 \n"
-"\n",
-str_is_default(gd), str_is_default(igd),
-str_is_default(gdp), str_is_default(igdp),
-str_is_default(igdplus), str_is_default(hausdorff));
+"\n");
 }
 
 static void
 do_file (const char *filename, double *reference, int reference_size,
-         int *nobj_p, const signed char * minmax)
+         int *nobj_p, const signed char * minmax, bool maximise_all_flag)
 {
     double *data = NULL;
     int *cumsizes = NULL;
     int nruns = 0;
     int nobj = *nobj_p;
 
-    int err = read_double_data (filename, &data, &nobj, &cumsizes, &nruns);
-    if (!filename) filename = stdin_name;
-    handle_read_data_error (err, filename);
+    handle_read_data_error(
+        read_double_data (filename, &data, &nobj, &cumsizes, &nruns), filename);
+    if (!filename)
+        filename = stdin_name;
 
     char *outfilename = NULL;
     FILE *outfile = stdout;
@@ -148,86 +146,65 @@ do_file (const char *filename, double *reference, int reference_size,
     /* Default minmax if not set yet.  */
     bool free_minmax = false;
     if (minmax == NULL) {
-        minmax = read_minmax (NULL, &nobj);
+        minmax = maximise_all_flag ? minmax_maximise(nobj) : minmax_minimise(nobj);
         free_minmax = true;
     }
 
-    const char * sep = "";
+    const char * sep = "\0";
     if (verbose_flag) {
         printf("# file: %s\n", filename);
         printf("# metrics (Euclidean distance) ");
-        if (gd) {
-            printf("GD");
-            sep = "\t";
-        }
-        if (igd) {
-            printf("%s", sep);
-            printf("IGD");
-            sep = "\t";
-        }
-        if (gdp) {
-            printf("%s", sep);
-            printf("GD_%d", exponent_p);
-            sep = "\t";
-        }
-        if (igdp) {
-            printf("%s", sep);
-            printf("IGD_%d", exponent_p);
-            sep = "\t";
-        }
-        if (igdplus) {
-            printf("%s", sep);
-            printf("IGD+");
-            sep = "\t";
-        }
-        if (hausdorff) {
-            printf("%s", sep);
-            printf("avg_Hausdorff");
-        }
+        /* This macro uses ## for comma elision in variadic macros. It should
+           use __VA_OPT__ in the future when more compilers support it:
+           https://gcc.gnu.org/onlinedocs/cpp/Variadic-Macros.html */
+#if defined(__clang__)
+#  pragma clang diagnostic push
+#  pragma clang diagnostic ignored "-Wgnu-zero-variadic-macro-arguments"
+#endif
+#define print_value_if(IF, WHAT, ...)                                          \
+        do {                                                                   \
+            if (IF) {                                                          \
+                fprintf(outfile, "%s" WHAT, sep, ## __VA_ARGS__);              \
+                sep = "\t";                                                    \
+            }                                                                  \
+        } while (0)
+
+        print_value_if(gd, "GD");
+        print_value_if(igd, "IGD");
+        print_value_if(gdp, "GD_%d", exponent_p);
+        print_value_if(igdp,"IGD_%d", exponent_p);
+        print_value_if(igdplus, "IGD+");
+        print_value_if(hausdorff, "avg_Hausdorff");
+#undef print_value_if
         printf("\n");
     }
 
-    int n;
-    int cumsize;
-    for (n = 0, cumsize = 0; n < nruns; cumsize = cumsizes[n], n++) {
-        double _no_warn_unused time_elapsed = 0;
+    for (int n = 0, cumsize = 0; n < nruns; cumsize = cumsizes[n], n++) {
+        _attr_maybe_unused double time_elapsed = 0;
         int size_a = cumsizes[n] - cumsize;
         const double *points_a = &data[nobj * cumsize];
         //Timer_start ();
-        sep = "";
+        sep = "\0";
 
-#define print_value_if(IF, VALUE)                                              \
+#define print_value_if(IF, FUN, ...)                                           \
         do {                                                                   \
             if (IF) {                                                          \
-                fprintf (outfile, "%s%-16.15g", sep, VALUE);                   \
+                fprintf (outfile, "%s" indicator_printf_format, sep,           \
+                         FUN(nobj, minmax, points_a, size_a, reference, reference_size, ## __VA_ARGS__)); \
                 sep = "\t";                                                    \
             }                                                                  \
-        } while(0)
+        } while (0)
 
-        print_value_if(gd,
-                       GD_minmax (nobj, minmax,
-                                  points_a, size_a,
-                                  reference, reference_size));
-        print_value_if(igd,
-                       IGD_minmax (nobj, minmax,
-                                   points_a, size_a,
-                                   reference, reference_size));
-        print_value_if(gdp,
-                       GD_p (nobj, minmax,
-                             points_a, size_a,
-                             reference, reference_size, exponent_p));
-        print_value_if(igdp,
-                       IGD_p (nobj, minmax,
-                              points_a, size_a,
-                              reference, reference_size, exponent_p));
-        print_value_if(igdplus,
-                       IGD_plus_minmax (nobj, minmax,
-                                        points_a, size_a,
-                                        reference, reference_size));
-        print_value_if(hausdorff,
-                       avg_Hausdorff_dist_minmax (nobj, minmax,
-                                                  points_a, size_a,
-                                                  reference, reference_size, exponent_p));
+        print_value_if(gd, GD_minmax);
+        print_value_if(igd, IGD_minmax);
+        print_value_if(gdp, GD_p, exponent_p);
+        print_value_if(igdp, IGD_p, exponent_p);
+        print_value_if(igdplus, IGD_plus_minmax);
+        print_value_if(hausdorff, avg_Hausdorff_dist_minmax, exponent_p);
+#undef print_value_if
+#if defined(__clang__)
+#  pragma clang diagnostic pop
+#endif
         //time_elapsed = Timer_elapsed_virtual ();
         fprintf(outfile, "\n");
         /* if (verbose_flag)  */
@@ -251,39 +228,39 @@ int main(int argc, char *argv[])
 {
     double *reference = NULL;
     int reference_size = 0;
-    int nobj = 0;
     const signed char *minmax = NULL;
+    bool maximise_all_flag = false;
+    int nobj = 0, tmp_nobj = 0;
 
     enum { GD_opt = 1000,
            IGD_opt, GD_p_opt, IGD_p_opt, IGD_plus_opt, hausdorff_opt};
-
     /* see the man page for getopt_long for an explanation of these fields */
-    static struct option long_options[] = {
-        {"gd",   no_argument,       NULL, GD_opt},
-        {"igd",   no_argument,       NULL, IGD_opt},
-        {"gd-p",   no_argument,       NULL, GD_p_opt},
-        {"igd-p",   no_argument,       NULL, IGD_p_opt},
-        {"igd-plus",   no_argument,       NULL, IGD_plus_opt},
-        {"hausdorff",   no_argument,       NULL, hausdorff_opt},
-        {"all",   no_argument,       NULL, 'a'},
-        {"exponent-p", required_argument,       NULL, 'p'},
-
+    static const char short_options[] = "hVvqap:Mr:s:o:";
+    static const struct option long_options[] = {
         {"help",       no_argument,       NULL, 'h'},
         {"version",    no_argument,       NULL, 'V'},
         {"verbose",    no_argument,       NULL, 'v'},
         {"quiet",      no_argument,       NULL, 'q'},
+        {"gd",         no_argument,       NULL, GD_opt},
+        {"igd",        no_argument,       NULL, IGD_opt},
+        {"gd-p",       no_argument,       NULL, GD_p_opt},
+        {"igd-p",      no_argument,       NULL, IGD_p_opt},
+        {"igd-plus",   no_argument,       NULL, IGD_plus_opt},
+        {"hausdorff",  no_argument,       NULL, hausdorff_opt},
+        {"all",        no_argument,       NULL, 'a'},
+        {"exponent-p", required_argument, NULL, 'p'},
+        {"maximise",   no_argument,       NULL, 'M'},
+        {"maximize",   no_argument,       NULL, 'M'},
         {"reference",  required_argument, NULL, 'r'},
         {"suffix",     required_argument, NULL, 's'},
         {"obj",        required_argument, NULL, 'o'},
-
         {NULL, 0, NULL, 0} /* marks end of list */
     };
-
     set_program_invocation_short_name(argv[0]);
 
     int opt;
     int longopt_index;
-    while (0 < (opt = getopt_long(argc, argv, "hVvqamr:us:o:",
+    while (0 < (opt = getopt_long(argc, argv, short_options,
                                   long_options, &longopt_index))) {
         switch (opt) {
           case 'p':
@@ -324,20 +301,25 @@ int main(int argc, char *argv[])
               hausdorff = true;
               break;
 
+        case 'M': // --maximise
+            maximise_all_flag = true;
+            break;
+
         case 'o': // --obj
-            minmax = read_minmax (optarg, &nobj);
-            if (minmax == NULL) {
-                fprintf(stderr, "%s: invalid argument '%s' for -o, --obj\n",
-                        program_invocation_short_name,optarg);
-                exit(EXIT_FAILURE);
-            }
+            minmax = parse_cmdline_minmax(minmax, optarg, &nobj);
             break;
 
         case 'r': // --reference
-            reference_size = read_reference_set (&reference, optarg, &nobj);
+            reference_size = read_reference_set(&reference, optarg, &tmp_nobj);
             if (reference == NULL || reference_size <= 0) {
                 errprintf ("invalid reference set '%s", optarg);
-                exit (EXIT_FAILURE);
+                exit(EXIT_FAILURE);
+            }
+            if (nobj == 0) {
+                nobj = tmp_nobj;
+            } else if (tmp_nobj != nobj) {
+                errprintf ("number of objectives in --obj (%d) and reference set (%d) do not match", nobj, tmp_nobj);
+                exit(EXIT_FAILURE);
             }
             break;
 
@@ -345,10 +327,6 @@ int main(int argc, char *argv[])
             suffix = optarg;
             break;
 
-        case 'V': // --version
-            version();
-            exit(EXIT_SUCCESS);
-
         case 'q': // --quiet
             verbose_flag = false;
             break;
@@ -357,31 +335,30 @@ int main(int argc, char *argv[])
             verbose_flag = true;
             break;
 
-        case '?':
-            // getopt prints an error message right here
-            fprintf(stderr, "Try `%s --help' for more information.\n",
-                    program_invocation_short_name);
-            exit(EXIT_FAILURE);
-        case 'h':
-            usage();
-            exit(EXIT_SUCCESS);
-        default: // should never happen
-            abort();
+        default:
+            default_cmdline_handler(opt);
         }
     }
 
+    if (!gd && !igd && !gdp && !igdp && !igdplus && !hausdorff) {
+        igdp = true; // Default is IGD_p.
+    }
+
     if (reference == NULL) {
         errprintf ("a reference set must be provided (--reference)");
-        exit (EXIT_FAILURE);
+        exit(EXIT_FAILURE);
     }
+    if (minmax == NULL) {
+        minmax = maximise_all_flag ? minmax_maximise(nobj) : minmax_minimise(nobj);
+    }
+    reference_size = filter_dominated_set(reference, nobj, reference_size, minmax);
 
     int numfiles = argc - optind;
     if (numfiles < 1) {/* Read stdin.  */
-        do_file (NULL, reference, reference_size, &nobj, minmax);
+        do_file (NULL, reference, reference_size, &nobj, minmax, maximise_all_flag);
     } else if (numfiles == 1) {
-        do_file (argv[optind], reference, reference_size, &nobj, minmax);
+        do_file (argv[optind], reference, reference_size, &nobj, minmax, maximise_all_flag);
     } else {
-        int k;
         /* FIXME: Calculate the nondominated front among all input
            files to use as reference set.  */
 #if 0
@@ -391,7 +368,7 @@ int main(int argc, char *argv[])
                                         minmax);
             write_sets (stderr, reference, nobj, &reference_size, 1);
         }
-        for (k = 0; k < numfiles; k++)
+        for (int k = 0; k < numfiles; k++)
             nondominatedfile_range (argv[optind + k], &maximum, &minimum, &nobj);
 
         if (verbose_flag >= 2) {
@@ -403,9 +380,11 @@ int main(int argc, char *argv[])
             printf ("\n");
         }
 #endif
-        for (k = 0; k < numfiles; k++)
-            do_file (argv[optind + k], reference, reference_size, &nobj, minmax);
+        for (int k = 0; k < numfiles; k++)
+            do_file (argv[optind + k], reference, reference_size, &nobj, minmax, maximise_all_flag);
     }
 
+    free(reference);
+    free((void*)minmax);
     return EXIT_SUCCESS;
 }
