@@ -14,13 +14,13 @@ nondom_init (size_t size)
 }
 
 static inline const double *
-force_agree_minimize (const double *points, int dim, int size,
-                      const signed char *minmax, const signed char agree _no_warn_unused)
+force_agree_minimize (const double *points, int dim_, int size,
+                      const signed char *minmax, _attr_maybe_unused const signed char agree)
 {
-    eaf_assert(agree != AGREE_MINIMISE);
-
+    assert(agree != AGREE_MINIMISE);
+    dimension_t dim = (dimension_t) dim_;
     bool no_copy = true;
-    for (int d = 0; d < dim; d++) {
+    for (dimension_t d = 0; d < dim; d++) {
         if (minmax[d] > 0) {
             no_copy = false;
             break;
@@ -32,8 +32,8 @@ force_agree_minimize (const double *points, int dim, int size,
     double *pnew = malloc(dim * size * sizeof(double));
     memcpy(pnew, points, dim * size * sizeof(double));
 
-    for (int d = 0; d < dim; d++) {
-        eaf_assert(minmax[d] != 0);
+    for (dimension_t d = 0; d < dim; d++) {
+        assert(minmax[d] != 0);
         if (minmax[d] > 0)
             for (int k = 0; k < size; k++)
                 pnew[k * dim + d] = -pnew[k * dim + d];
@@ -51,41 +51,33 @@ static inline int compare_x_asc_y_asc (const void *p1, const void *p2)
                              ((y1 < y2) ? -1 : ((y1 > y2) ? 1 : 0)));
 }
 
-/* When find_dominated_p == true, then stop as soon as one dominated point is
-   found and return its position.
-
-   When find_dominated_p == false, store which points are nondominated in nondom
-   and return the number of nondominated points.
-
-*/
-static inline int
-find_nondominated_set_2d_(const double *points, int size,
-                          const signed char *minmax, const signed char agree,
-                          bool *nondom, bool find_dominated_p, bool keep_weakly)
+static inline
+const double ** generate_sorted_pp_2d(const double *points, int size)
 {
-    if (size < 2)
-        return size;
-
-    // FIXME: We should do this before reaching this function and remove minmax
-    // and agree from this function.
-    const double *pp = force_agree_minimize (points, 2, size, minmax, agree);
-
     const double **p = malloc (size * sizeof(double *));
     for (int k = 0; k < size; k++)
-        p[k] = pp + 2 * k;
+        p[k] = points + 2 * k;
 
     qsort(p, size, sizeof(*p), &compare_x_asc_y_asc);
-    int n_dominated = 0, k = 0, j = 1;
+    return p;
+}
+
+/*
+   Stop as soon as one dominated point is found and return its position (or SIZE_MAX if no dominated point found).
+*/
+static inline int
+find_dominated_2d_(const double *points, int size, const bool keep_weakly)
+{
+    const double **p = generate_sorted_pp_2d(points, size);
+    // In this context, it means "no dominated solution found".
+    int pos_first_dom = -1;
+    int k = 0, j = 1;
     do {
         while (j < size && p[j][1] >= p[k][1]) {
             if (!keep_weakly || p[j][0] != p[k][0] || p[j][1] != p[k][1]) {
-                if (find_dominated_p) {
-                    // In this context, it means "position of the first dominated solution found".
-                    n_dominated = (int)((p[j] - pp) / 2);
-                    goto early_end;
-                }
-                nondom[j] = false;
-                n_dominated++;
+                // In this context, it means "position of the first dominated solution found".
+                pos_first_dom = (int)((p[j] - points) / 2);
+                goto early_end;
             }
             j++;
         }
@@ -93,22 +85,48 @@ find_nondominated_set_2d_(const double *points, int size,
         j++;
     } while (j < size);
 
-    if (find_dominated_p) {
-        // In this context, it means "no dominated solution found".
-        n_dominated = -1;
-        goto early_end;
-    }
-
-    bool * nondom_new = malloc(size * sizeof(bool));
-    for (k = 0; k < size; k++)
-        nondom_new[(p[k] - pp) / 2] = nondom[k];
-    memcpy(nondom, nondom_new, size * sizeof(bool));
-    free (nondom_new);
 early_end:
     free(p);
-    if (pp != points)
-        free((void*)pp);
-    return n_dominated;
+    return pos_first_dom;
+}
+
+
+/*
+   Store which points are nondominated in nondom and return the number of
+   nondominated points.
+*/
+
+static inline int
+find_nondominated_set_2d_(const double * points, int size, bool * nondom,
+                          const bool keep_weakly)
+{
+    // When compiling with -O3, GCC is able to create two versions of this loop
+    // and move keep_weakly out.
+    const double **p = generate_sorted_pp_2d(points, size);
+    int n_nondom = size, k = 0, j = 1;
+    do {
+        while (j < size && p[j][1] >= p[k][1]) {
+            if (!keep_weakly || p[j][0] != p[k][0] || p[j][1] != p[k][1]) {
+                nondom[j] = false;
+                n_nondom--;
+            }
+            j++;
+        }
+        k = j;
+        j++;
+    } while (j < size);
+
+    if (n_nondom < size) {
+        /* nondom[] uses the order in p[], which is sorted, so we have to map it to
+           the original order in points. */
+        bool * nondom_new = malloc(size * sizeof(bool));
+        memcpy(nondom_new, nondom, size * sizeof(bool));
+        for (k = 0; k < size; k++)
+            nondom[(p[k] - points) / 2] = nondom_new[k];
+        free(nondom_new);
+    }
+    free(p);
+    return n_nondom;
 }
 
 /* When find_dominated_p == true, then stop as soon as one dominated point is
@@ -119,15 +137,30 @@ early_end:
 
 */
 static inline int
-find_nondominated_set_ (const double *points, int dim, int size,
-                        const signed char *minmax, const signed char agree,
-                        bool *nondom, bool find_dominated_p, bool keep_weakly)
+find_nondominated_set_ (const double * points, int dim_, int size,
+                        const signed char * minmax, signed char agree,
+                        bool * nondom, bool find_dominated_p, bool keep_weakly)
 {
-    if (dim == 2)
-        return find_nondominated_set_2d_(points, size, minmax, agree,
-                                       nondom, find_dominated_p, keep_weakly);
-    int j, k, d;
+    if (size < 2)
+        return size;
 
+    ASSUME(dim_ >= 2);
+    dimension_t dim = (dimension_t) dim_;
+
+    if (dim == 2) {
+        const double *pp = force_agree_minimize (points, 2, size, minmax, agree);
+        int res = (find_dominated_p)
+            ? find_dominated_2d_(pp, size, keep_weakly)
+            : find_nondominated_set_2d_(pp, size, nondom, keep_weakly);
+        if (pp != points)
+            free((void*)pp);
+        return res;
+    }
+
+    if (agree == AGREE_NONE)
+        agree = (signed char) check_all_minimize_maximize(minmax, dim);
+
+    int j, k;
     for (k = 0; k < size - 1; k++) {
         for (j = k + 1; j < size; j++) {
 
@@ -145,26 +178,35 @@ find_nondominated_set_ (const double *points, int dim, int size,
             /* FIXME: Do not handle agree here, assume that objectives
                have been fixed already to agree on
                minimization/maximization.  */
-            if (agree < 0) {
-                for (d = 0; d < dim; d++) {
-                    j_leq_k = j_leq_k && (pj[d] <= pk[d]);
-                    k_leq_j = k_leq_j && (pk[d] <= pj[d]);
-                }
-            } else if (agree > 0) {
-                for (d = 0; d < dim; d++) {
-                    j_leq_k = j_leq_k && (pj[d] >= pk[d]);
-                    k_leq_j = k_leq_j && (pk[d] >= pj[d]);
-                }
-            } else {
-                for (d = 0; d < dim; d++) {
-                    if (minmax[d] < 0) {
-                        j_leq_k = j_leq_k && (pj[d] <= pk[d]);
-                        k_leq_j = k_leq_j && (pk[d] <= pj[d]);
-                    } else if (minmax[d] > 0) {
-                        j_leq_k = j_leq_k && (pj[d] >= pk[d]);
-                        k_leq_j = k_leq_j && (pk[d] >= pj[d]);
-                    }
-                }
+            switch (agree) {
+              case AGREE_NONE:
+                  for (dimension_t d = 0; d < dim; d++) {
+                      if (minmax[d] < 0) {
+                          j_leq_k = j_leq_k && (pj[d] <= pk[d]);
+                          k_leq_j = k_leq_j && (pk[d] <= pj[d]);
+                      } else if (minmax[d] > 0) {
+                          j_leq_k = j_leq_k && (pj[d] >= pk[d]);
+                          k_leq_j = k_leq_j && (pk[d] >= pj[d]);
+                      }
+                  }
+                  break;
+
+              case AGREE_MINIMISE:
+                  for (dimension_t d = 0; d < dim; d++) {
+                      j_leq_k = j_leq_k && (pj[d] <= pk[d]);
+                      k_leq_j = k_leq_j && (pk[d] <= pj[d]);
+                  }
+                  break;
+
+              case AGREE_MAXIMISE:
+                  for (dimension_t d = 0; d < dim; d++) {
+                      j_leq_k = j_leq_k && (pj[d] >= pk[d]);
+                      k_leq_j = k_leq_j && (pk[d] >= pj[d]);
+                  }
+                  break;
+
+              default:
+                  unreachable();
             }
 
             // k is removed if it is weakly dominated by j (unless keep_weakly == FALSE).
@@ -172,7 +214,7 @@ find_nondominated_set_ (const double *points, int dim, int size,
             // j is removed if it is dominated by k.
             nondom[j] = (!k_leq_j || j_leq_k);
 
-            eaf_assert(nondom[k] || nondom[j]); /* both cannot be removed.  */
+            assert(nondom[k] || nondom[j]); /* both cannot be removed.  */
 
             if (find_dominated_p && (!nondom[k] || !nondom[j])) {
                 return nondom[k] ? j : k;
@@ -184,7 +226,7 @@ find_nondominated_set_ (const double *points, int dim, int size,
 
     int new_size = 0;
     for (k = 0; k < size; k++)
-        if (nondom[k]) new_size++;
+        new_size += (int) nondom[k];
     return new_size;
 }
 
@@ -193,7 +235,6 @@ find_dominated_point (const double *points, int dim, int size,
                       const signed char *minmax)
 {
     bool *nondom = nondom_init (size);
-
     int pos = find_nondominated_set_ (points, dim, size, minmax,
                                       AGREE_NONE, nondom,
                                       /* find_dominated_p = */true,
@@ -235,35 +276,76 @@ get_nondominated_set (double **pareto_set_p,
                       const double *points, int dim, int size,
                       const signed char *minmax)
 {
-    bool *nondom  = nondom_init(size);
+    ASSUME(dim >= 1);
+    ASSUME(dim <= 32);
+    ASSUME(size > 0);
+
+    bool *nondom = nondom_init(size);
     int new_size = find_nondominated_set (points, dim, size, minmax, nondom);
-
-    DEBUG2 (
-        fprintf (stderr, "# size\tnondom\tdom\n");
-        fprintf (stderr, "  %d\t%d\t%d\n",
-                 size, new_size, size - new_size);
-        );
-
-    if (new_size > size) {/* This can't happen.  */
+    if (new_size > size || new_size <= 0) { /* This can't happen.  */
+        fprintf (stderr,
+                 "# size\tnondom\tdom\n"
+                 "  %d\t%d\t%d\n",  size, new_size, size - new_size);
         fatal_error ("%s:%d: a bug happened: new_size > old_size!\n",
                      __FILE__, __LINE__);
     }
 
     double *pareto_set = malloc (sizeof (double) * new_size * dim);
-    int n, k;
-    // FIXME: We could use new_size to stop earlier.
-    for (n = 0, k = 0; n < size; n++) {
-        if (!nondom[n]) continue;
-        memcpy(&pareto_set[dim * k], &points[dim * n], sizeof(points[0]) * dim);
-        k++;
+    if (new_size < size) {
+        int n = 0, k = 0;
+        do {
+            while (!nondom[n]) n++;
+            memcpy(pareto_set + dim * k, points + dim * n, sizeof(points[0]) * dim);
+            k++;
+            n++;
+        } while (k < new_size);
+    } else {
+        // Nothing is dominated. Copy everything in one go.
+        memcpy(pareto_set, points, sizeof(points[0]) * dim * size);
     }
-    eaf_assert (k == new_size);
     free (nondom);
     *pareto_set_p = pareto_set;
     return new_size;
 }
 
-_no_warn_unused static bool *
+static inline int
+filter_dominated_set (double *points, int dim, int size,
+                      const signed char *minmax)
+{
+    ASSUME(dim >= 1);
+    ASSUME(dim <= 32);
+    ASSUME(size > 0);
+    if (size == 1)
+        return 1;
+
+    bool *nondom = nondom_init(size);
+    int new_size = find_nondominated_set (points, dim, size, minmax, nondom);
+    if (new_size > size || new_size <= 0) { /* This can't happen.  */
+        fprintf (stderr,
+                 "# size\tnondom\tdom\n"
+                 "  %d\t%d\t%d\n",  size, new_size, size - new_size);
+
+        fatal_error ("%s:%d: a bug happened: new_size > old_size!\n",
+                     __FILE__, __LINE__);
+    }
+
+    if (new_size < size) {
+        int k = 0;
+        while (nondom[k]) k++; // Find first dominated.
+        int n = k;
+        while (k < new_size) {
+            do {
+                n++;
+            } while (!nondom[n]); // Find next nondominated.
+            memcpy(points + dim * k, points + dim * n, sizeof(points[0]) * dim);
+            k++;
+        }
+    }
+    free (nondom);
+    return new_size;
+}
+
+_attr_maybe_unused static bool *
 is_nondominated (const double * data, int nobj, int npoint, const bool * maximise, bool keep_weakly)
 {
     bool * nondom = nondom_init(npoint);
@@ -316,7 +398,7 @@ normalise (double *points, int dim, int size,
     free (diff);
 }
 
-_no_warn_unused static void
+_attr_maybe_unused static void
 agree_normalise (double *data, int nobj, int npoint,
                  const bool * maximise,
                  const double lower_range, const double upper_range,
